@@ -304,7 +304,9 @@
 #if defined (CONFIG_LCD_TD035TTEA1)
 	/* One Stop Display QVGA TD035TTEA1 320x240 */
 # define PANEL_NAME		"OSB LCD Landscape QVGA"
-# define PEL_CLOCK_EST		(8650*1000)     /* MHz/4/8/8.65 */
+//# define PEL_CLOCK_EST		(8650*1000)     /* MHz/4/8/8.65 */
+# define PEL_CLOCK_EST		(8000*1000)     /* MHz/4/8/8.65 */
+//# define PEL_CLOCK_EST		(8650*1000)     /* MHz/4/8/8.65 */
 # define PEL_CLOCK_DIV		CLOCK_TO_DIV(PEL_CLOCK_EST, HCLK)
 # define PEL_CLOCK		(HCLK/PEL_CLOCK_DIV)
 # define PEL_WIDTH		(320)
@@ -322,6 +324,7 @@
 # define INVERT_HSYNC
 # define INVERT_VSYNC
 # define INVERT_PIXEL_CLOCK
+//# define INVERT_OE
 #endif
 
 	/* Inverse timing calculations for peculiar Sharp panels
@@ -339,6 +342,7 @@
 #define HSYNC_WIDTH		(((PANEL_TIMING0 >> 8) & 0x7f) + 1)
 #define VSYNC_WIDTH		(((PANEL_TIMING1 >> 10) & 0x3f) + 1)
 
+#define P_INVERT_OE		(PANEL_TIMING2  & (1<<14))
 #define P_INVERT_PIXEL_CLOCK	(PANEL_TIMING2  & (1<<13))
 #define P_INVERT_HSYNC		(PANEL_TIMING2  & (1<<12))
 #define P_INVERT_VSYNC		(PANEL_TIMING2  & (1<<11))
@@ -352,6 +356,10 @@
 #  define INVERT_PIXEL_CLOCK
 # endif
 
+# if P_INVERT_OE
+#  define INVERT_OE
+# endif
+
 # if P_INVERT_HSYNC
 #  define INVERT_HSYNC
 # endif
@@ -390,6 +398,23 @@
 #define BW	(1<<4)
 #define LCDEN	(1<<0)
 
+	// Decode macros
+#define D_PEL_CLOCK_DIV		((CLCDC_TIMING2 & 0x1f) + 2)
+#define D_PEL_WIDTH		(((CLCDC_TIMING2 >> 16) & 0x3ff) + 1)
+#define D_PEL_HEIGHT		((CLCDC_TIMING1 & 0x1ff) + 1)
+#define D_HBP			(((CLCDC_TIMING0 >> 24) & 0xff) + 1)
+#define D_HFP			(((CLCDC_TIMING0 >> 16) & 0xff) + 1)
+#define D_VBP			((CLCDC_TIMING1 >> 24) & 0xff)
+#define D_VFP			((CLCDC_TIMING1 >> 16) & 0xff)
+
+#define D_HSW			(((CLCDC_TIMING0 >> 8) & 0x7f) + 1)
+#define D_VSW			(((CLCDC_TIMING1 >> 10) & 0x3f) + 1)
+
+#define D_IOE			(CLCDC_TIMING2  & (1<<14))
+#define D_IPC			(CLCDC_TIMING2  & (1<<13))
+#define D_IHS			(CLCDC_TIMING2  & (1<<12))
+#define D_IVS			(CLCDC_TIMING2  & (1<<11))
+
 
 #define CB_BUFFER ((PEL_WIDTH*PEL_HEIGHT*BIT_DEPTH)/8)
 
@@ -456,7 +481,14 @@ static void _msleep (int ms)
 
 static void clcdc_init (void)
 {
+#if defined (CONFIG_LCD_RETAIN_SPLASH)
+  /* The alignment here has to do with the kernel.  It didn't like
+     receiving a block of memory that wasn't MB boundary aligned.  Not
+     sure why and I don't care for now. */
+  buffer = alloc_uncached_top_retain (CB_BUFFER, 1024*1024);
+#else
   buffer = alloc_uncached (CB_BUFFER, 1024*1024);
+#endif
 
 #if defined (USE_FILL)
 
@@ -498,6 +530,9 @@ static void clcdc_init (void)
   CLCDC_TIMING1 = VBP (VERT_BP) | VFP (VERT_FP) | VSW (VSYNC_WIDTH)
     | LPP (PEL_HEIGHT);
   CLCDC_TIMING2   = CPL
+#if defined (INVERT_OE)
+    | IOE
+#endif
 #if defined (INVERT_PIXEL_CLOCK)
     | IPC
 #endif
@@ -568,6 +603,7 @@ static void clcdc_init (void)
 
 static void clcdc_release (void)
 {
+#if !defined (CONFIG_LCD_RETAIN_SPLASH)
   /* *** This sequence, as well as that which is in the initialization
      function, should be better documented. */
 
@@ -591,6 +627,7 @@ static void clcdc_release (void)
      DRV_CLCDC_RELEASE as the driver is likely to disable clocks
      making the registers unavailable.  */
   DRV_CLCDC_RELEASE;
+#endif
 }
 
 #if !defined (CONFIG_SMALL)
@@ -599,14 +636,21 @@ static void clcdc_report (void)
   unsigned long clk = HCLK/((CLCDC_TIMING2 & 0x1f) + 2);
   printf ("  clcd:   buffer 0x%p  red %d<<%d  green %d<<%d  blue %d<<%d\n",
 	  buffer, 5, RED_SHIFT, 5, GREEN_SHIFT, 5,  BLUE_SHIFT);
-  printf ("          ctrl 0x%lx\n", CLCDC_CTRL);
+  printf ("          ctrl 0x%lx", CLCDC_CTRL);
+  if (clk < 1000000)
+    printf ("  clk %ldHz", clk);
+  else
+    printf ("  clk %ld.%03ldMHz",
+	    clk/(1000*1000), (clk/1000)%1000);
+  printf (" [ " PANEL_NAME " ]\n");
   printf ("          timing0 0x%08lx  timing1 0x%08lx  timing2 0x%08lx\n",
 	  CLCDC_TIMING0, CLCDC_TIMING1, CLCDC_TIMING2);
-  if (clk < 1000000)
-    printf ("          clk %ldHz\n", clk);
-  else
-    printf ("          clk %ld.%03ldMHz\n",
-	    clk/(1000*1000), (clk/1000)%1000);
+  printf ("          hpels %ld hbp %ld hsw %ld hfp %ld "
+	  "ioe %d ipc %d ihs %d ivs %d\n",
+	  D_PEL_WIDTH, D_HBP, D_HSW, D_HFP,
+	  D_IOE != 0, D_IPC != 0, D_IHS != 0, D_IVS != 0);
+  printf ("          vpels %ld vbp %ld vsw %ld vfp %ld\n",
+	  D_PEL_HEIGHT, D_VBP, D_VSW, D_VFP);
 }
 #endif
 
@@ -791,6 +835,8 @@ int cmd_clcdc (int argc, const char** argv)
     _msleep (20);		/* Wait 20ms for digital signals  */
     CLCDC_CTRL      |= PWR;	/* Apply power */
 
+    _msleep (20);		/* *** Hack */
+
     DRV_CLCDC_WAKE;
 
     /* *** FIXME: this value is calculable based on the LCD controller
