@@ -45,10 +45,9 @@
 
      This conversion is done by reading all of the partition entries,
      looking for the partition table's own entry.  When it is found,
-     the driver checks the length of the block for a match with the
-     NOR flash erase block size.  If there is a match when the data is
-     swapped, it is assumed that the system is reading a BE partition
-     table.
+     the driver compares the swapped start address of the directory
+     partition with the address we're reading.  If both are in the
+     same erase block, we know we have a swapped directory.
 
      For simplicity and general correctness, this is done every time
      this driver activates.  The table could be cached, but it is
@@ -116,6 +115,28 @@
    the same area, when skips are ignored, will not match.  This is on
    purpose.
 
+   fis_directory_entries
+   ---------------------
+
+   Except in the prescan call, the search through the directory is
+   ultimately limited by the number of possible entries in the
+   partition containing the directory.  The end_of_table () call is
+   likely to limit the search even more, but this is an optimization
+   introduced here in APEX.  In prescan, we depend on the end_of_table
+   () call and the extent of the region passed via the environment or
+   alias as "fis-drv".
+
+   Deleted Entries and the End of the Table
+   ----------------------------------------
+
+   The Redboot will write 0xff to the start of a partition entry when
+   it deletes it.  This is peculiar since the only way to write 0xff
+   is to write the whole partition, so why not simply remove the
+   partition instead.  Duh.
+
+   The end of the table is marked by 0xff's in the first two bytes of
+   the descriptor.
+
 */
 
 #include <config.h>
@@ -153,6 +174,7 @@ struct fis_descriptor {
 };
 
 static int fis_directory_swap;	/* Set for a byte swapped directory */
+static int fis_directory_entries; /* Computed from the size of the partition */
 
 struct fis_skip_descriptor {
   char magic[4];		/* 's' 'k' 'i' 'p' */
@@ -184,9 +206,15 @@ static inline const char* block_driver (void)
   return lookup_alias_or_env ("fis-drv", CONFIG_DRIVER_FIS_BLOCKDEVICE);
 }
 
-static int end_of_table (struct fis_descriptor* descriptor)
+static inline int deleted_entry (struct fis_descriptor* partition)
 {
-  return descriptor->start == 0xffffffff;
+	/* Erased entry -- dumb, but that's RedBoot */
+  return partition->name[0] == 0xff;
+}
+
+static inline int end_of_table (struct fis_descriptor* partition)
+{
+  return (*(u16*) &partition->name[0]) == 0xffff;
 }
 
 static int compare_skips (const void* _a, const void* _b)
@@ -204,31 +232,55 @@ static void prescan_directory (struct descriptor_d* d)
   unsigned long eraseblocksize = 0;
 
   fis_directory_swap = 0;
+  fis_directory_entries = 0;
 
   descriptor_query (d, QUERY_START, &start);
   descriptor_query (d, QUERY_SIZE, &size);
 
-  PRINTF ("%s: start %lx size %ld\n", __FUNCTION__, start, size);
+  PRINTF ("%s: start %lx size %ld start 0x%lx index 0x%x\n",
+	  __FUNCTION__, start, size, d->start, d->index);
   if (size == 0)
     return;			/* Unable to perform queries */
 
   d->driver->seek (d, 0, SEEK_SET);
 
   while (1) {
-    struct fis_descriptor descriptor;
-    if (d->driver->read (d, &descriptor, sizeof (descriptor))
-	!= sizeof (descriptor))
+    struct fis_descriptor partition;
+    if (d->driver->read (d, &partition, sizeof (partition))
+	!= sizeof (partition))
       break;
-    if (end_of_table (&descriptor))
+    if (end_of_table (&partition))
       break;
+    if (deleted_entry (&partition))
+      continue;
 
-    if (strnicmp (descriptor.name, "fis directory", 16) == 0) {
+    if (strnicmp (partition.name, "fis directory", 16) == 0) {
+      /* Read the erase block size where we need it since it is
+	 possible we're crossing boundaries within flash where the
+	 erase block size changes. */
       descriptor_query (d, QUERY_ERASEBLOCKSIZE, &eraseblocksize);
-      PRINTF ("%s: length %lx  eb %lx  eb_swapped %lx\n",
-	      __FUNCTION__, descriptor.length, eraseblocksize,
-	      swab32 (eraseblocksize));
-      if (descriptor.length == swab32 (eraseblocksize))
+      /* The original implementation of this check was to compare the
+	 the FIS directory partition entry's length field with
+	 eraseblocksize swapped.  This, however, assumed that the
+	 directory was always the full size of an erase block,
+	 something which is not (nor ever was) true.  So we changed
+	 the test to something more robust.  Now, we check that the
+	 partition table start address is in the same erase block from
+	 which we are reading. */
+      PRINTF ("%s: partition.start 0x%lx (0x%lx)  &partition 0x%lx"
+	      "  mask 0x%lx\n",
+	      __FUNCTION__, partition.start, swab32 (partition.start),
+	      start + d->start + d->index, ~(eraseblocksize - 1));
+      if ((swab32 (partition.start) & ~(eraseblocksize - 1))
+	  == ((unsigned long) (start + d->start + d->index)
+	      & ~(eraseblocksize - 1))) {
 	fis_directory_swap = 1;
+	fis_directory_entries = swab32 (partition.length)/sizeof (partition);
+      }
+      else
+	fis_directory_entries = partition.length/sizeof (partition);
+      PRINTF ("%s: %d entries, swapped %d\n",
+	      __FUNCTION__, fis_directory_entries, fis_directory_swap);
       break;
     }
   }
@@ -236,7 +288,7 @@ static void prescan_directory (struct descriptor_d* d)
 }
 
 static const char* map_region (struct descriptor_d* d,
-			       struct fis_descriptor* descriptor)
+			       struct fis_descriptor* partition)
 {
   static char sz[64];
   unsigned long start = 0;
@@ -244,7 +296,7 @@ static const char* map_region (struct descriptor_d* d,
 
   snprintf (sz, sizeof (sz), "%s:0x%08lx+0x%08lx",
 	    d->driver_name,
-	    descriptor->start - start, descriptor->length);
+	    partition->start - start, partition->length);
   return sz;
 }
 
@@ -252,6 +304,7 @@ static int fis_open (struct descriptor_d* d)
 {
   int result;
   struct descriptor_d fis_d;
+  int cEntries;
 
   if (d->c != 1 || d->iRoot != 1)
     ERROR_RETURN (ERROR_BADPARTITION, "path must include partition");
@@ -264,7 +317,7 @@ static int fis_open (struct descriptor_d* d)
 
   PRINTF ("%s: d@0x%lx+0x%lx\n", __FUNCTION__, d->start, d->length);
 
-  while (1) {
+  for (cEntries = fis_directory_entries ; cEntries--; ) {
     struct fis_descriptor partition;
     int skip = 0;
     size_t cbSkip = 0;
@@ -276,6 +329,8 @@ static int fis_open (struct descriptor_d* d)
     }
     if (end_of_table (&partition))
       break;
+    if (deleted_entry (&partition))
+      continue;
     if (strnicmp (d->pb[0], partition.name, sizeof (partition.name)))
       continue;
 
@@ -441,6 +496,7 @@ static void fis_report (void)
 {
   int result;
   struct descriptor_d d;
+  int cEntries;
 
   if (   (result = parse_descriptor (block_driver (), &d))
       || (result = open_descriptor (&d)))
@@ -450,7 +506,7 @@ static void fis_report (void)
 
   printf ("  fis:\n");
 
-  while (1) {
+  for (cEntries = fis_directory_entries; cEntries--; ) {
     struct fis_descriptor partition;
     int i;
 
@@ -459,6 +515,8 @@ static void fis_report (void)
       break;
     if (end_of_table (&partition))
       break;
+    if (deleted_entry (&partition))
+      continue;
 
     if (fis_directory_swap) {
       partition.start = swab32 (partition.start);
